﻿using Microsoft.Extensions.Logging;
using ShipyardContracts.BindingModels;
using ShipyardContracts.BusinessLogicsContracts;
using ShipyardContracts.SearchModels;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ShipyardView
{
    public partial class FormImplementer : Form
    {
        private readonly ILogger _logger;
        private readonly IImplementerLogic _logic;
        private int? _id;
        public int Id { set { _id = value; } }
        public FormImplementer(ILogger<FormDetail> logger, IImplementerLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
        }

        private void FormImplementer_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                try
                {
                    _logger.LogInformation("Получение исполнителя");
                    var view = _logic.ReadElement(new ImplementerSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        textBoxFIO.Text = view.ImplementerFIO;
                        textBoxExperience.Text = view.WorkExperience.ToString();
                        textBoxPassword.Text = view.Password;
                        textBoxQualification.Text = view.Qualification.ToString();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка получения исполнителя");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxFIO.Text))
            {
                MessageBox.Show("Заполните ФИО", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение исполнителя");
            try
            {
                var model = new ImplementerBindingModel
                {
                    Id = _id ?? 0,
                    ImplementerFIO = textBoxFIO.Text,
                    Password = textBoxPassword.Text,
                    WorkExperience = Convert.ToInt32(textBoxExperience.Text),
                    Qualification = Convert.ToInt32(textBoxQualification.Text),
                };
                var operationResult = _id.HasValue ? _logic.Update(model) :
                _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения компонента");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
