﻿using Microsoft.EntityFrameworkCore;
using ShipyardContracts.BindingModels;
using ShipyardContracts.SearchModels;
using ShipyardContracts.StoragesContracts;
using ShipyardContracts.ViewModels;
using ShipyardDataBaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ShipyardDataBaseImplement.Implements
{
    public class MessageInfoStorage : IMessageInfoStorage
    {
        public MessageInfoViewModel? GetElement(MessageInfoSearchModel model)
        {
            if (string.IsNullOrEmpty(model.MessageId))
            {
                return null;
            }
            using var context = new ShipyardDataBase();
            return context.Messages
             .FirstOrDefault(x => x.MessageId == model.MessageId)?
            .GetViewModel;
        }

        public List<MessageInfoViewModel> GetFilteredList(MessageInfoSearchModel model)
        {
            using var context = new ShipyardDataBase();
            if (model.ClientId.HasValue && model.Page.HasValue && model.PageCount.HasValue)
            {
                return context.Messages
                    .Where(x => x.ClientId == model.ClientId)
                    .Skip(model.PageCount.Value * model.Page.Value).Take(model.PageCount.Value)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.Page.HasValue && model.PageCount.HasValue)
            {
                return context.Messages
                    .Skip(model.PageCount.Value * model.Page.Value).Take(model.PageCount.Value)
                    .Select(x => x.GetViewModel).ToList();
            }
            return new();
        }

        public List<MessageInfoViewModel> GetFullList()
        {
            using var context = new ShipyardDataBase();
            return context.Messages
            .Select(x => x.GetViewModel)
            .ToList();
        }

        public MessageInfoViewModel? Insert(MessageInfoBindingModel model)
        {
            using var context = new ShipyardDataBase();
            var newMessage= Message.Create(model);
            if (newMessage == null)
            {
                return null;
            }
            context.Messages.Add(newMessage);
            context.SaveChanges();
            return newMessage.GetViewModel;
        }

        public MessageInfoViewModel? Update(MessageInfoBindingModel model)
        {
            using var context = new ShipyardDataBase();
            var message = context.Messages.FirstOrDefault(x => x.MessageId ==
           model.MessageId);
            if (message == null)
            {
                return null;
            }
            message.Update(model);
            context.SaveChanges();
            return message.GetViewModel;
        }
    }
}
