﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using ShipyardFileImplement.Models;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace ShipyardFileImplement
{
    public class DataFileSingleton
    {
        private static DataFileSingleton? instance;
        private readonly string DetailFileName = "Detail.xml";
        private readonly string OrderFileName = "Order.xml";
        private readonly string ShipFileName = "Ship.xml";
        private readonly string ClientFileName = "Client.xml";
        private readonly string MessageFileName = "Message.xml";
        private readonly string ImplementerFileName = "Implementer.xml";

        private readonly string ShopFileName = "Shop.xml";
        public List<Detail> Details { get; private set; }
        public List<Order> Orders { get; private set; }
        public List<Ship> Ships { get; private set; }
        public List<Client> Clients { get; private set; }
        public List<Implementer> Implementers { get; private set; }

        public List<Message> Messages { get; private set; }
        public List<Shop> Shops { get; private set; }
        public static DataFileSingleton GetInstance()
        {
            if (instance == null)
            {
                instance = new DataFileSingleton();
            }
            return instance;
        }
        public void SaveDetails() => SaveData(Details, DetailFileName,"Details", x => x.GetXElement);
        public void SaveShips() => SaveData(Ships, ShipFileName,"Ships", x => x.GetXElement);
        public void SaveOrders() => SaveData(Orders, OrderFileName, "Orders", x=> x.GetXElement);
        public void SaveClients() => SaveData(Clients, ClientFileName, "Clients", x => x.GetXElement);
        public void SaveImplementers() => SaveData(Implementers, ImplementerFileName, "Implementers", x => x.GetXElement);

        public void SaveMessages() => SaveData(Messages, MessageFileName, "Messages", x => x.GetXElement);
        public void SaveShops() => SaveData(Shops, ShopFileName, "Shops", x => x.GetXElement);
        private DataFileSingleton()
        {
            Details = LoadData(DetailFileName, "Detail", x => Detail.Create(x)!)!;
            Ships = LoadData(ShipFileName, "Ship", x => Ship.Create(x)!)!;
            Orders = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
            Clients = LoadData(ClientFileName, "Client", x => Client.Create(x)!)!;
            Implementers = LoadData(ImplementerFileName, "Implementer", x => Implementer.Create(x)!)!;
            Messages = LoadData(MessageFileName, "Message", x => Message.Create(x)!)!;
            Shops = LoadData(ShopFileName, "Shop", x => Shop.Create(x)!)!;
        }
        private static List<T>? LoadData<T>(string filename, string xmlNodeName,
        Func<XElement, T> selectFunction)
        {
            if (File.Exists(filename))
            {
                return XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList();
            }
            return new List<T>();
        }
        private static void SaveData<T>(List<T> data, string filename, string
        xmlNodeName, Func<T, XElement> selectFunction)
        {
            if (data != null)
            {
                new XDocument(new XElement(xmlNodeName,
               data.Select(selectFunction).ToArray())).Save(filename);
            }
        }
    }
}
