﻿using ShipyardContracts.BindingModels;
using ShipyardContracts.SearchModels;
using ShipyardContracts.StoragesContracts;
using ShipyardContracts.ViewModels;
using ShipyardFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ShipyardFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;
        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders.Select(x => GetViewModel(x)).ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (model.Id.HasValue)
            {
                return source.Orders
                              .Where(x => x.Id == model.Id)
                              .Select(x => x.GetViewModel)
                              .ToList();
            }
            else if (model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                return source.Orders
                              .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                              .Select(x => x.GetViewModel)
                              .ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return source.Orders
                              .Where(x => x.ClientId == model.ClientId)
                              .Select(x => x.GetViewModel)
                              .ToList();
            }
            else if (model.Status.HasValue && model.ImplementerId.HasValue)
            {
                return source.Orders
                    .Where(x => x.Status == model.Status && x.ImplementerId == model.ImplementerId)
                    .Select(x => x.GetViewModel).ToList();
            }
            else if (model.Status.HasValue)
            {
                return source.Orders
                        .Where(x => x.Status == model.Status)
                        .Select(x => GetViewModel(x)).ToList();
            }
            return new();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (model.Id.HasValue)
            {
                return source.Orders.FirstOrDefault(x =>
                (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
            }
            else if (model.ImplementerId.HasValue) 
            {
                return source.Orders.FirstOrDefault(x =>
                (x.ImplementerId == model.ImplementerId))?.GetViewModel;
            }
            return null;
            
        }
        private OrderViewModel GetViewModel(Order order)
        {
            var viewModel = order.GetViewModel;
            var ship = source.Ships.FirstOrDefault(x => x.Id == order.ShipId);
            if (ship != null)
            {
                viewModel.ShipName = ship.ShipName;
            }
            return viewModel;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            source.Orders.Add(newOrder);
            source.SaveOrders();
            return GetViewModel(newOrder);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            source.SaveOrders();
            return GetViewModel(order);
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Orders.Remove(element);
                source.SaveOrders();
                return GetViewModel(element);
            }
            return null;
        }
    }
}
