﻿using ShipyardContracts.BindingModels;
using ShipyardContracts.ViewModels;
using ShipyardDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace ShipyardFileImplement.Models
{
    public class Message : IMessageInfoModel
    {
        public string MessageId { get; private set; } = string.Empty;

        public int? ClientId { get; private set; }

        public string SenderName { get; private set; } = string.Empty;

        public DateTime DateDelivery { get; private set; } = DateTime.Now;

        public string Subject { get; private set; } = string.Empty;

        public string Body { get; private set; } = string.Empty;
        public bool IsRead { get; private set; } = false;
        public string? AnswerText { get; private set; }
        public static Message? Create(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new()
            {
                Body = model.Body,
                Subject = model.Subject,
                DateDelivery = model.DateDelivery,
                SenderName = model.SenderName,
                ClientId = model.ClientId,
                MessageId = model.MessageId
            };
        }

        public static Message? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new()
            {
                Body = element.Attribute("Body")!.Value,
                Subject = element.Attribute("Subject")!.Value,
                DateDelivery = Convert.ToDateTime(element.Attribute("DateDelivery")!.Value),
                SenderName = element.Attribute("SenderName")!.Value,
                ClientId = Convert.ToInt32(element.Attribute("ClientId")!.Value),
                MessageId = element.Attribute("MessageId")!.Value,
            };
        }

        public void Update(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            IsRead = model.IsRead;
            AnswerText = model.AnswerText;
        }
        public MessageInfoViewModel GetViewModel => new()
        {
            Body = Body,
            Subject = Subject,
            DateDelivery = DateDelivery,
            SenderName = SenderName,
            ClientId = ClientId,
            MessageId = MessageId,
            IsRead=IsRead,
            AnswerText=AnswerText
        };

        public XElement GetXElement => new("MessageInfo",
            new XAttribute("Subject", Subject),
            new XAttribute("Body", Body),
            new XAttribute("ClientId", ClientId),
            new XAttribute("MessageId", MessageId),
            new XAttribute("SenderName", SenderName),
            new XAttribute("DateDelivery", DateDelivery),
            new XAttribute("IsRead", IsRead),
            new XAttribute("AnswerText", AnswerText)
            );
    }
}
