﻿using ShipyardContracts.BindingModels;
using ShipyardContracts.ViewModels;
using ShipyardDataModels.Enums;
using ShipyardDataModels.Models;
using System.Xml.Linq;

namespace ShipyardFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int ShipId { get; private set; }
        public int ClientId { get; private set; }
        public int? ImplementerId { get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                ShipId = model.ShipId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
            };
        }

        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            var order = new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShipId = Convert.ToInt32(element.Element("ShipId")!.Value),
                ClientId=Convert.ToInt32(element.Element("ClientId")!.Value),
                ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                DateCreate = DateTime.ParseExact(element.Element("DateCreate")!.Value, "G", null),
            };
            DateTime.TryParse(element.Element("DateImplement")!.Value, out DateTime dateImpl);
            order.DateImplement = dateImpl;

            if (!Enum.TryParse(element.Element("Status")!.Value, out OrderStatus status))
            {
                return null;
            }
            order.Status = status;
            return order;
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            ShipId = ShipId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            Id = Id,
            Status = Status,
        };

        public XElement GetXElement => new("Order",
          new XAttribute("Id", Id),
          new XElement("ShipId", ShipId),
          new XElement("ClientId", ClientId),
          new XElement("ImplementerId", ImplementerId),
          new XElement("Count", Count.ToString()),
          new XElement("Sum", Sum.ToString()),
          new XElement("Status", Status.ToString()),
          new XElement("DateCreate", DateCreate.ToString()),
          new XElement("DateImplement", DateImplement.ToString()));

        
    }
}

