﻿using ShipyardContracts.BindingModels;
using ShipyardContracts.SearchModels;
using ShipyardContracts.StoragesContracts;
using ShipyardContracts.ViewModels;
using ShipyardListImplement;
using ShipyardListImplement_.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ShipyardListImplement_.Implements
{
    public class MessageInfoStorage : IMessageInfoStorage
    {
        private readonly DataListSingleton _source;
        public MessageInfoStorage() 
        {
            _source=DataListSingleton.GetInstance();
        }
        public MessageInfoViewModel? GetElement(MessageInfoSearchModel model)
        {
            if (string.IsNullOrEmpty(model.MessageId))
            {
                return null;
            }
            foreach (var message in _source.Messages)
            {
                if(message.MessageId==model.MessageId)
                {
                    return message.GetViewModel;
                }
            }
            return null;
        }

        public List<MessageInfoViewModel> GetFilteredList(MessageInfoSearchModel model)
        {
            var result = new List<MessageInfoViewModel>();
            if (string.IsNullOrEmpty(model.MessageId))
            {
                return result;
            }
            foreach (var message in _source.Messages)
            {
                if (message.ClientId==model.ClientId)
                {
                    result.Add(message.GetViewModel);
                }
            }
            return result;
        }

        public List<MessageInfoViewModel> GetFullList()
        {
            var result = new List<MessageInfoViewModel>();
            foreach (var message in _source.Messages)
            {
                result.Add(message.GetViewModel);
            }
            return result;
        }

        public MessageInfoViewModel? Insert(MessageInfoBindingModel model)
        {
            var newMessage = Message.Create(model);
            if (newMessage == null)
            {
                return null;
            }
            _source.Messages.Add(newMessage);
            return newMessage.GetViewModel;
        }

        public MessageInfoViewModel? Update(MessageInfoBindingModel model)
        {
            foreach (var message in _source.Messages)
            {
                if (message.MessageId == model.MessageId)
                {
                    message.Update(model);
                    return message.GetViewModel;
                }
            }
            return null;
        }
    }
}
